package com.V10lator.SignClock;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.logging.Logger;
import org.bukkit.World;
import org.bukkit.Material;
import org.bukkit.block.Block;
import org.bukkit.event.Event;
import org.bukkit.plugin.PluginDescriptionFile;
import org.bukkit.plugin.PluginManager;
import org.bukkit.plugin.java.JavaPlugin;
import org.bukkit.Server;

public class SignClock extends JavaPlugin
{
  private Logger log = Logger.getLogger("Minecraft");
  private String name;
  private File saveFile;
  private final SignClockBlockListener blockListener = new SignClockBlockListener(this);
  private final SignClockPlayerListener playerListener = new SignClockPlayerListener(this);
  private final HashMap<Block, SClock> signs = new HashMap<Block, SClock>();

  public void onEnable()
  {
	Server server = getServer();
    PluginManager pm = server.getPluginManager();
    pm.registerEvent(Event.Type.SIGN_CHANGE, blockListener, Event.Priority.Normal, this);
    pm.registerEvent(Event.Type.REDSTONE_CHANGE, blockListener, Event.Priority.Normal, this);
    pm.registerEvent(Event.Type.BLOCK_BREAK, blockListener, Event.Priority.Normal, this);
    pm.registerEvent(Event.Type.PLAYER_INTERACT, playerListener, Event.Priority.Normal, this);
    
    PluginDescriptionFile pdfFile = this.getDescription();
    this.name = pdfFile.getName();
    saveFile = new File("plugins/"+this.name+"/data.sav");
    
    if(!checkFiles())
    {
      pm.disablePlugin(this);
      return;
    }
    
    this.info2log("v" + pdfFile.getVersion() + " enabled");
    this.loadSigns();
  }
  
  public void onDisable()
  {
	if(signs.size() != 0)
	  this.saveSigns();
    this.info2log("disabled");
  }

  public void info2log(String text)
  {
    log.info("[" + this.name + "] " + text + ".");
  }
  
  public void addSign(Block block, SClock sign)
  {
    //add Sign to hashmap
    signs.put(block, sign);
	    
    saveSigns();
  }
	    
  public boolean isSignClock(Block block)
  {
    return signs.containsKey(block);
  }
	    
  public void removeSign(Block block)
  {
    signs.remove(block);
    saveSigns();
  }
	    
  public SClock getSignClock(Block block)
  {
    if(!isSignClock(block))
      return null;
	return signs.get(block);
  }
	    
  private boolean checkFiles()
  {
    //create|check Plugin folder
	        File pluginDir = new File("plugins/"+this.name);

	        if(!pluginDir.exists())
	        {
	         if(pluginDir.mkdir()) {
	         this.info2log("info: create "+this.name+" plugin folder");
	         }
	         else {
	         this.info2log("error: can't create "+this.name+" plugin folder, Signs will not be saved!");
	         return false;
	         }
	        }
	        
	        //create|check saveFile
	        if(!saveFile.exists())
	try {
	saveFile.createNewFile();

	this.info2log("info: save file created");
	} catch (IOException e) {
	this.info2log("error: can't create "+this.name+" save file, Signs will not be saved!");
	e.printStackTrace();
	return false;
	}

	return true;
	    }
	    
	    
	    private void saveSigns() {
	     //open savefile
	try {
	FileWriter writer = new FileWriter(saveFile);

	//write data
	for(SClock sign : this.signs.values())
	{
	//write Signs to file
	writer.write(sign.getWorld() + "," + sign.getX() + "," + sign.getY() + "," + sign.getZ() + "," + sign.getStyle() + "\n");
	}

	writer.close();

	} catch (IOException e) {
	this.info2log("error: can't write save data");
	e.printStackTrace();
	}
	    }
	    
	    private int loadSigns() {
	     int count = 0;

	     try {
	BufferedReader reader = new BufferedReader(new FileReader(saveFile));
	Block block = null;
	World world = null;
//	String[] lines = null;
//	String[][] newLines = new String[2][4];

	//read Data
	while(reader.ready())
	{
    String out = reader.readLine();
    String[] result = out.split(",");
	SClock sign = new SClock();
	sign.setWorld(result[0]);
	sign.setStyle(Integer.parseInt (result[4]));
	sign.setX(Integer.parseInt (result[1]));
	sign.setY(Integer.parseInt (result[2]));
	sign.setZ(Integer.parseInt (result[3]));
	sign.setEnvironment("NORMAL");
	sign.setFlank(false);
	sign.setState(0);

	if(sign.getEnvironment() != null)
	world = getServer().createWorld(sign.getWorld(),World.Environment.valueOf(sign.getEnvironment()));
	else
	world = getServer().getWorld(sign.getWorld());

	//get Block
	if(world != null)
	{
	block = world.getBlockAt(sign.getX(), sign.getY(), sign.getZ());
	}
	else
	{
	break;
	}

	//put sign in array if it a sign
	this.removeSign(block);
	if(block != null && block.getType() == Material.SIGN_POST || block.getType() == Material.SIGN || block.getType() == Material.WALL_SIGN)
	{
      signs.put(block, sign);
	  count++;
	}
	}

	reader.close();
	} catch (FileNotFoundException e) {
	this.info2log("error: can't read save data");
	e.printStackTrace();
	} catch (IOException e) {
	this.info2log("error: can't read save data");
	e.printStackTrace();
	}

	this.info2log(count + " Clocks resetted");
	return count;
  }
}